package org.jfedor.touchscreenbroadcaster;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.View.OnKeyListener;
import android.view.View.OnTouchListener;

public class TouchscreenBroadcasterActivity extends Activity {
    
    int PORT = 20124;
    byte MESG_TOUCH = 0x01;
    byte MESG_KEY = 0x02;
    
    TBView mTBView;
    DatagramSocket mSocket;
    PowerManager powerManager;
    WakeLock wakeLock;
    InetAddress address;
    long counter = 0;
    
    private class TBTouchListener implements OnTouchListener {

        @Override
        public boolean onTouch(View v, MotionEvent event) {
            byte[] bytes = null;
            int action = event.getAction();
            if ((action == MotionEvent.ACTION_DOWN) || (action == MotionEvent.ACTION_MOVE)) {
                float x = event.getX();
                float y = event.getY();
                ByteBuffer bb = ByteBuffer.allocate(8+1+4+8);
                bb.putLong(counter++);
                bb.put(MESG_TOUCH);
                bb.putInt(action);
                FloatBuffer fb = bb.asFloatBuffer();
                fb.put(x);
                fb.put(y);
                bytes = bb.array();
            } else if ((action == MotionEvent.ACTION_UP) || (action == MotionEvent.ACTION_CANCEL)) {
                ByteBuffer bb = ByteBuffer.allocate(8+1+4);
                bb.putLong(counter++);
                bb.put(MESG_TOUCH);
                bb.putInt(action);
                bytes = bb.array();
            }
            
            if (bytes != null) {
                DatagramPacket packet = new DatagramPacket(bytes, bytes.length, address, PORT);
                try {
                    mSocket.send(packet);
                } catch (IOException e) {
                }
            }
            
            return true;
        }
    
    }
    
    private class TBKeyListener implements OnKeyListener {

        @Override
        public boolean onKey(View v, int keyCode, KeyEvent event) {
            byte[] bytes = null;
            ByteBuffer bb = ByteBuffer.allocate(8+1+4+4);
            bb.putLong(counter++);
            bb.put(MESG_KEY);
            bb.putInt(event.getAction());
            bb.putInt(keyCode);
            bytes = bb.array();
            DatagramPacket packet = new DatagramPacket(bytes, bytes.length, address, PORT);
            try {
                mSocket.send(packet);
            } catch (IOException e) {
            }
            return true;
        }
        
    }
    
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        
        powerManager = (PowerManager) getSystemService(Context.POWER_SERVICE);
        wakeLock = powerManager.newWakeLock(PowerManager.SCREEN_DIM_WAKE_LOCK, "Touchscreen Broadcaster");
        
        mTBView = new TBView(this, null);
        setContentView(mTBView);
        mTBView.setOnTouchListener(new TBTouchListener());
        mTBView.setOnKeyListener(new TBKeyListener());
        mTBView.requestFocus();
    }
    
    @Override
    protected void onStart() {
        super.onStart();
        
        try {
            mSocket = new DatagramSocket();
            address = InetAddress.getByName("255.255.255.255");
        } catch (SocketException e) {
        } catch (UnknownHostException e) {
        }
        
        wakeLock.acquire();
    }

    @Override
    protected void onStop() {
        super.onStop();
        mSocket.close();
        mSocket = null;
        wakeLock.release();
    }
}